@@ -21,10 +21,10 @@ from gi.repository.Gio import FILE_ATTRIBUTE_STANDARD_NAME, \
 
 from gettext import gettext as _
 from threading import Thread
-from time import time, sleep
+from time import time
 
 from lollypop.inotify import Inotify
-from lollypop.define import App, Type
+from lollypop.define import App, Type, ScanType
 from lollypop.objects import Track, Album
 from lollypop.sqlcursor import SqlCursor
 from lollypop.tagreader import TagReader
@@ -67,30 +67,31 @@ class CollectionScanner(GObject.GObject, TagReader):
             self.__inotify = None
         App().albums.update_max_count()
 
-    def update(self, uris=[], saved=True):
+    def update(self, scan_type, uris=[]):
         """
             Update database
+            @param scan_type as ScanType
             @param uris as [str]
-            @param saved as bool
         """
+        App().lookup_action("update_db").set_enabled(False)
         # Stop previous scan
-        if self.is_locked():
+        if self.is_locked() and scan_type != ScanType.EPHEMERAL:
             self.stop()
-            GLib.timeout_add(250, self.update, uris, saved)
+            GLib.timeout_add(250, self.update, scan_type, uris)
         else:
             self.__disable_compilations = not App().settings.get_value(
                 "show-compilations")
 
-            if not uris:
+            if scan_type == ScanType.FULL:
                 uris = App().settings.get_music_uris()
-                if not uris:
-                    return
-
+            if not uris:
+                return
             # Register to progressbar
-            App().window.container.progress.add(self)
-            App().window.container.progress.set_fraction(0, self)
+            if scan_type != ScanType.EPHEMERAL:
+                App().window.container.progress.add(self)
+                App().window.container.progress.set_fraction(0, self)
             # Launch scan in a separate thread
-            self.__thread = Thread(target=self.__scan, args=(uris, saved))
+            self.__thread = Thread(target=self.__scan, args=(scan_type, uris))
             self.__thread.daemon = True
             self.__thread.start()
 
@@ -114,13 +115,16 @@ class CollectionScanner(GObject.GObject, TagReader):
             Update progress bar status
             @param scanned items as int, total items as int
         """
-        App().window.container.progress.set_fraction(current / total, self)
+        GLib.idle_add(App().window.container.progress.set_fraction,
+                      current / total,
+                      self)
 
     def __finish(self, modifications):
         """
             Notify from main thread when scan finished
             @param modifications as bool
         """
+        App().lookup_action("update_db").set_enabled(True)
         App().window.container.progress.set_fraction(1.0, self)
         self.stop()
         self.emit("scan-finished", modifications)
@@ -145,11 +149,12 @@ class CollectionScanner(GObject.GObject, TagReader):
                 self.__inotify.add_monitor(d)
 
     @profile
-    def __get_objects_for_uris(self, uris):
+    def __get_objects_for_uris(self, scan_type, uris):
         """
             Get all tracks and dirs in uris
+            @param scan_type as ScanType
             @param uris as string
-            @return (tracks as [mtime: int, uri: str], dirs as [uri: str])
+            @return (tracks [mtimes: int, uri: str], dirs as [uri: str])
         """
         files = []
         dirs = []
@@ -178,6 +183,7 @@ class CollectionScanner(GObject.GObject, TagReader):
                         else:
                             mtime = get_mtime(info)
                             files.append((mtime, child_uri))
+                # Only happens if files passed as args
                 else:
                     mtime = get_mtime(info)
                     files.append((mtime, uri))
@@ -188,36 +194,43 @@ class CollectionScanner(GObject.GObject, TagReader):
         return (files, dirs)
 
     @profile
-    def __scan(self, uris, saved):
+    def __scan(self, scan_type, uris):
         """
             Scan music collection for music files
+            @param scan_type as ScanType
             @param uris as [str]
-            @param saved as bool
             @thread safe
         """
-        if self.__history is None:
+        if scan_type != ScanType.EPHEMERAL and self.__history is None:
             self.__history = History()
 
-        (files, dirs) = self.__get_objects_for_uris(uris)
+        (files, dirs) = self.__get_objects_for_uris(scan_type, uris)
 
-        new_tracks = self.__scan_files(files, saved)
-        self.__add_monitor(dirs)
+        if scan_type == ScanType.NEW_FILES:
+            db_uris = App().tracks.get_uris(uris)
+        else:
+            db_uris = App().tracks.get_uris()
+        new_tracks = self.__scan_files(files, db_uris, scan_type)
 
-        GLib.idle_add(self.__finish, new_tracks and saved)
+        if scan_type != ScanType.EPHEMERAL:
+            self.__add_monitor(dirs)
+            GLib.idle_add(self.__finish, new_tracks)
 
-        if not saved and self.__thread is not None:
+        if scan_type == ScanType.EPHEMERAL:
             self.__play_new_tracks(new_tracks)
 
-        del self.__history
-        self.__history = None
+        if scan_type != ScanType.EPHEMERAL:
+            del self.__history
+            self.__history = None
 
-    def __scan_to_handle(self, f):
+    def __scan_to_handle(self, uri):
         """
             Check if file has to be handle by scanner
             @param f as Gio.File
             @return bool
         """
         try:
+            f = Gio.File.new_for_uri(uri)
             # Scan file
             if is_pls(f):
                 # Handle playlist
@@ -230,92 +243,57 @@ class CollectionScanner(GObject.GObject, TagReader):
             Logger.error("CollectionScanner::__scan_to_handle(): %s" % e)
         return False
 
-    def __scan_add(self, uri, mtime):
-        """
-            Add audio file to database
-            @param uri as str
-            @param mtime as float
-        """
-        try:
-            Logger.debug("Adding file: %s" % uri)
-            self.__add2db(uri, mtime)
-            SqlCursor.allow_thread_execution(App().db)
-        except Exception as e:
-            Logger.error("CollectionScanner::__scan_add(add): "
-                         "%s, %s" % (e, uri))
-
-    def __scan_del(self, uri):
-        """
-            Delete file from DB
-            @param uri as str
-        """
-        try:
-            self.__del_from_db(uri)
-            SqlCursor.allow_thread_execution(App().db)
-        except Exception as e:
-            Logger.error("CollectionScanner::__scan_del: %s" % e)
-
     @profile
-    def __scan_files(self, files, saved):
+    def __scan_files(self, files, db_uris, scan_type):
         """
             Scan music collection for new audio files
-            @param files as [(int, str)]
+            @param files as [str]
+            @param db_uris as [str]
+            @param scan_type as ScanType
             @return new track uris as [str]
             @thread safe
         """
+        SqlCursor.add(App().db)
         i = 0
-        to_add = []
+        # New tracks present in collection
         new_tracks = []
         # Get mtime of all tracks to detect which has to be updated
-        mtimes = App().tracks.get_mtimes()
-        # Get uris of all tracks to detect which has to be deleted
-        to_delete = App().tracks.get_uris()
-        count = len(files) + len(to_delete)
-        SqlCursor.add(App().db)
+        db_mtimes = App().tracks.get_mtimes()
+        count = len(files) + 1
         try:
-            while files:
+            # Scan new files
+            for (mtime, uri) in files:
                 # Handle a stop request
-                if self.__thread is None:
-                    raise Exception("Scan cancelled")
+                if self.__thread is None and scan_type != ScanType.EPHEMERAL:
+                    raise Exception("Scan add cancelled")
                 try:
-                    (mtime, uri) = files.pop(0)
-                    f = Gio.File.new_for_uri(uri)
-                    already_in_db = uri in to_delete
-                    if already_in_db:
-                        to_delete.remove(uri)
-                        i += 1
-                    if mtime > mtimes.get(uri, 0):
-                        handled = self.__scan_to_handle(f)
-                        if handled:
-                            if already_in_db:
-                                self.__scan_del(uri)
-                            # On first scan, we want file mtime
-                            mtime = int(time()) if mtimes else mtime
-                            # If not saved, use 0 as mtime, easy delete on quit
-                            to_add.append((uri, mtime if saved else 0))
-                            new_tracks.append(uri)
-                            i -= 1
-                    # Previous code (<= 0.9.910) was reading mtime in this loop
-                    # Since we are looping faster, seems it's makes python
-                    # lagging. If someone has a better idea
-                    elif i % 1000:
-                        sleep(0.00001)
+                    if not self.__scan_to_handle(uri):
+                        continue
+                    if mtime > db_mtimes.get(uri, 0):
+                        # If not saved, use 0 as mtime, easy delete on quit
+                        if scan_type == ScanType.EPHEMERAL:
+                            mtime = 0
+                        # Do not use mtime if not intial scan
+                        elif db_mtimes:
+                            mtime = int(time())
+                        Logger.debug("Adding file: %s" % uri)
+                        self.__add2db(uri, mtime)
+                        SqlCursor.allow_thread_execution(App().db)
+                        new_tracks.append(uri)
                 except Exception as e:
                     Logger.error(
-                               "CollectionScanner:: __scan_files: % s" % e)
+                               "CollectionScanner:: __scan_add_files: % s" % e)
                 i += 1
-                GLib.idle_add(self.__update_progress, i, count)
-            # This files are not in collection anymore
-            # We do this before __scan_add() to populate history
-            if saved:
-                for uri in to_delete:
-                    i += 1
-                    GLib.idle_add(self.__update_progress, i, count)
-                    self.__scan_del(uri)
-            for (uri, mtime) in to_add:
-                i += 1
-                GLib.idle_add(self.__update_progress, i, count)
-                self.__scan_add(uri, mtime)
+                self.__update_progress(i, count)
+            if scan_type != ScanType.EPHEMERAL and self.__thread is not None:
+                for uri in db_uris:
+                    # Handle a stop request
+                    if self.__thread is None:
+                        raise Exception("Scan del cancelled")
+                    f = Gio.File.new_for_uri(uri)
+                    if not f.query_exists():
+                        self.__del_from_db(uri)
+                        SqlCursor.allow_thread_execution(App().db)
         except Exception as e:
             Logger.warning("CollectionScanner:: __scan_files: % s" % e)
         SqlCursor.commit(App().db)
@@ -373,9 +351,25 @@ class CollectionScanner(GObject.GObject, TagReader):
 
         Logger.debug("CollectionScanner::add2db(): Restore stats")
         # Restore stats
-        (track_pop, track_rate, track_ltime, album_mtime,
-         track_loved, album_loved, album_pop, album_rate) = self.__history.get(
-            name, duration)
+        track_id = App().tracks.get_id_by_uri(uri)
+        if track_id is None:
+            basename = f.get_basename()
+            track_id = App().tracks.get_id_by_basename_duration(basename,
+                                                                duration)
+        # Restore from history
+        if self.__history is None:
+            (track_pop, track_rate, track_ltime,
+             album_mtime, track_loved, album_loved,
+             album_pop, album_rate) = (0, 0, 0, 0, False, False, 0, 0)
+        elif track_id is None:
+            (track_pop, track_rate, track_ltime,
+             album_mtime, track_loved, album_loved,
+             album_pop, album_rate) = self.__history.get(name, duration)
+        # Delete track and restore from it
+        else:
+            (track_pop, track_rate, track_ltime,
+             album_mtime, track_loved, album_loved,
+             album_pop, album_rate) = self.__del_from_db(uri)
         # If nothing in stats, use track mtime
         if album_mtime == 0:
             album_mtime = mtime
@@ -429,28 +423,24 @@ class CollectionScanner(GObject.GObject, TagReader):
         """
             Delete track from db
             @param uri as str
+            @return (popularity, ltime, mtime,
+                     loved album, album_popularity)
         """
         try:
-            f = Gio.File.new_for_uri(uri)
-            name = f.get_basename()
             track_id = App().tracks.get_id_by_uri(uri)
             album_id = App().tracks.get_album_id(track_id)
             genre_ids = App().tracks.get_genre_ids(track_id)
             album_artist_ids = App().albums.get_artist_ids(album_id)
             artist_ids = App().tracks.get_artist_ids(track_id)
-            popularity = App().tracks.get_popularity(track_id)
-            rate = App().tracks.get_rate(track_id)
-            ltime = App().tracks.get_ltime(track_id)
-            mtime = App().tracks.get_mtime(track_id)
-            loved_track = App().tracks.get_loved(track_id)
-            duration = App().tracks.get_duration(track_id)
-            album_popularity = App().albums.get_popularity(album_id)
+            track_pop = App().tracks.get_popularity(track_id)
+            track_rate = App().tracks.get_rate(track_id)
+            track_ltime = App().tracks.get_ltime(track_id)
+            album_mtime = App().tracks.get_mtime(track_id)
+            track_loved = App().tracks.get_loved(track_id)
+            album_pop = App().albums.get_popularity(album_id)
             album_rate = App().albums.get_rate(album_id)
-            loved_album = App().albums.get_loved(album_id)
+            album_loved = App().albums.get_loved(album_id)
             uri = App().tracks.get_uri(track_id)
-            self.__history.add(name, duration, popularity, rate,
-                               ltime, mtime, loved_track, loved_album,
-                               album_popularity, album_rate)
             App().tracks.remove(track_id)
             App().tracks.clean(track_id)
             cleaned = App().albums.clean(album_id)
@@ -460,16 +450,17 @@ class CollectionScanner(GObject.GObject, TagReader):
                               album_id, True)
             for artist_id in album_artist_ids + artist_ids:
                 cleaned = App().artists.clean(artist_id)
-                # Force update even if not cleaned as artist may
-                # have been removed from a selected genre
-                GLib.idle_add(self.emit, "artist-updated",
-                              artist_id, False)
+                if cleaned:
+                    GLib.idle_add(self.emit, "artist-updated",
+                                  artist_id, False)
             for genre_id in genre_ids:
                 cleaned = App().genres.clean(genre_id)
                 if cleaned:
                     SqlCursor.commit(App().db)
                     GLib.idle_add(self.emit, "genre-updated",
                                   genre_id, False)
+            return (track_pop, track_rate, track_ltime, album_mtime,
+                    track_loved, album_loved, album_pop, album_rate)
         except Exception as e:
             Logger.error("CollectionScanner::__del_from_db: %s" % e)
 
@@ -500,8 +491,9 @@ class CollectionScanner(GObject.GObject, TagReader):
                     album_artist_ids = new_artist_ids
                 App().albums.set_artist_ids(album_id, new_artist_ids)
         # Update UI based on previous artist calculation
-        for artist_id in album_artist_ids:
-            GLib.idle_add(self.emit, "artist-updated", artist_id, add)
+        if App().albums.get_tracks_count(album_id) > 1:
+            for artist_id in album_artist_ids:
+                GLib.idle_add(self.emit, "artist-updated", artist_id, add)
         # Update album genres
         for genre_id in genre_ids:
             App().albums.add_genre(album_id, genre_id)
