@@ -15,7 +15,7 @@ from gi.repository import Gtk, Gdk, GLib, Gio, Pango
 from gettext import gettext as _
 from gettext import ngettext as ngettext
 
-from lollypop.define import App
+from lollypop.define import App, ScanType
 from lollypop.logger import Logger
 from lollypop.database import Database
 from lollypop.database_history import History
@@ -668,7 +668,15 @@ class SettingsDialog:
         self.__settings_dialog.hide()
         self.__settings_dialog.destroy()
         if set(previous) != set(uris):
-            App().scanner.update()
+            to_delete = [uri for uri in previous if uri not in uris]
+            if to_delete:
+                # We need to do a full scan
+                App().scanner.update(ScanType.FULL)
+            else:
+                # Only scan new folders
+                to_scan = [uri for uri in uris if uri not in previous]
+                if to_scan:
+                    App().scanner.update(ScanType.NEW_FILES, to_scan)
 
     def __test_lastfm_connection(self, result, fm):
         """
@@ -806,7 +814,7 @@ class SettingsDialog:
             App().db = Database()
             App().window.container.show_genres(
                 App().settings.get_value("show-genres"))
-            App().scanner.update()
+            App().scanner.update(ScanType.FULL)
             self.__progress.get_toplevel().set_deletable(True)
 
 
