@@ -41,7 +41,17 @@ class ContextWidget(Gtk.EventBox):
         grid = Gtk.Grid()
         grid.show()
 
-        if not isinstance(object, Disc):
+        if isinstance(object, Disc):
+            play_button = Gtk.Button.new_from_icon_name(
+                "media-playback-start-symbolic",
+                Gtk.IconSize.BUTTON)
+            play_button.connect("clicked", self.__on_play_button_clicked)
+            play_button.get_style_context().add_class("dim-button")
+            play_button.set_tooltip_text(_("Play"))
+            play_button.set_margin_end(2)
+            play_button.show()
+            grid.add(play_button)
+        else:
             edit_button = Gtk.Button.new_from_icon_name(
                 "document-properties-symbolic",
                 Gtk.IconSize.BUTTON)
@@ -73,7 +83,7 @@ class ContextWidget(Gtk.EventBox):
             queue_button.show()
             grid.add(queue_button)
         else:
-            add_to_playback = True
+            add = True
             string = _("Add to current playlist")
             icon = "list-add-symbolic"
             # Search album in player
@@ -81,27 +91,27 @@ class ContextWidget(Gtk.EventBox):
                 album = object.album
             else:
                 album = object
-            player_album = None
+            player_album_track_ids = []
             for _album in App().player.albums:
                 if album.id == _album.id:
-                    player_album = _album
-                    break
-            if player_album is not None:
-                if player_album.track_ids.sort() == object.track_ids.sort():
-                    add_to_playback = False
-                    string = _("Remove from current playlist")
-                    icon = "list-remove-symbolic"
-            playback_button = Gtk.Button.new_from_icon_name(
+                    player_album_track_ids += _album.track_ids
+            if len(set(player_album_track_ids) &
+                    set(object.track_ids)) == len(object.track_ids):
+                add = False
+                string = _("Remove from current playlist")
+                icon = "list-remove-symbolic"
+            current_playlist_button = Gtk.Button.new_from_icon_name(
                 icon,
                 Gtk.IconSize.BUTTON)
-            playback_button.connect("clicked",
-                                    self.__on_playback_button_clicked,
-                                    add_to_playback)
-            playback_button.get_style_context().add_class("dim-button")
-            playback_button.set_tooltip_text(string)
-            playback_button.set_margin_end(2)
-            playback_button.show()
-            grid.add(playback_button)
+            current_playlist_button.connect(
+                                    "clicked",
+                                    self.__on_current_playlist_button_clicked,
+                                    add)
+            current_playlist_button.get_style_context().add_class("dim-button")
+            current_playlist_button.set_tooltip_text(string)
+            current_playlist_button.set_margin_end(2)
+            current_playlist_button.show()
+            grid.add(current_playlist_button)
 
         playlist_button = Gtk.Button.new_from_icon_name("view-list-symbolic",
                                                         Gtk.IconSize.BUTTON)
@@ -135,6 +145,24 @@ class ContextWidget(Gtk.EventBox):
         """
         return True
 
+    def __on_play_button_clicked(self, button):
+        """
+            Play disc
+            @param button as Gtk.Button
+        """
+        try:
+            App().player.clear_albums()
+            album = Album(self.__object.album.id)
+            # search wanted disc
+            for disc in album.discs:
+                if disc.number == self.__object.number:
+                    album.set_tracks(disc.tracks)
+                    break
+            App().player.add_album(album)
+            App().player.load(album.tracks[0])
+        except Exception as e:
+            Logger.error("ContextWidget::__on_play_button_clicked(): %s", e)
+
     def __on_queue_button_clicked(self, button, add_to_queue):
         """
             Add or remove track to/from queue
@@ -146,27 +174,30 @@ class ContextWidget(Gtk.EventBox):
                 if add_to_queue:
                     App().player.append_to_queue(track_id, False)
                 else:
-                    App().player.del_from_queue(track_id, False)
+                    App().player.remove_from_queue(track_id, False)
         elif isinstance(self.__object, Track):
             if add_to_queue:
                 App().player.append_to_queue(self.__object.id, False)
             else:
-                App().player.del_from_queue(self.__object.id, False)
+                App().player.remove_from_queue(self.__object.id, False)
         App().player.emit("queue-changed")
         self.hide()
 
-    def __on_playback_button_clicked(self, button, add_to_playback):
+    def __on_current_playlist_button_clicked(self, button, add):
         """
             Add to/remove from current playback
             @param button as Gtk.Button
-            @param add_to_playback as bool
+            @param add as bool
         """
         if isinstance(self.__object, Disc):
-            album = self.__object.album
+            album = self.__object.album.clone(True)
+            album.set_discs([self.__object])
         else:
-            album = self.__object
-        if add_to_playback:
+            album = self.__object.clone(True)
+        if add:
             App().player.add_album(album)
+        elif isinstance(self.__object, Disc):
+            App().player.remove_disc(self.__object, album.id)
         else:
             App().player.remove_album_by_id(album.id)
         self.hide()
